# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "BudgetTypeEnum",
    },
)


class BudgetTypeEnum(proto.Message):
    r"""Describes Budget types."""

    class BudgetType(proto.Enum):
        r"""Possible Budget types.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            STANDARD (2):
                Budget type for standard Google Ads usage.
                Caps daily spend at two times the specified
                budget amount. Full details:
                https://support.google.com/google-ads/answer/6385083
            FIXED_CPA (4):
                Budget type with a fixed cost-per-acquisition (conversion).
                Full details:
                https://support.google.com/google-ads/answer/7528254

                This type is only supported by campaigns with
                AdvertisingChannelType.DISPLAY (excluding
                AdvertisingChannelSubType.DISPLAY_GMAIL),
                BiddingStrategyType.TARGET_CPA and PaymentMode.CONVERSIONS.
            SMART_CAMPAIGN (5):
                Budget type for Smart Campaign. Full details:
                https://support.google.com/google-ads/answer/7653509

                This type is only supported by campaigns with
                AdvertisingChannelType.SMART and
                AdvertisingChannelSubType.SMART_CAMPAIGN.
            LOCAL_SERVICES (6):
                Budget type for Local Services Campaign. Full details:
                https://support.google.com/localservices/answer/7434558

                This type is only supported by campaigns with
                AdvertisingChannelType.LOCAL_SERVICES.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        STANDARD = 2
        FIXED_CPA = 4
        SMART_CAMPAIGN = 5
        LOCAL_SERVICES = 6


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "AdDestinationTypeEnum",
    },
)


class AdDestinationTypeEnum(proto.Message):
    r"""Container for enumeration of Google Ads destination types."""

    class AdDestinationType(proto.Enum):
        r"""Enumerates Google Ads destination types

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                The value is unknown in this version.
            NOT_APPLICABLE (2):
                Ads that don't intend to drive users off from
                ads to other destinations
            WEBSITE (3):
                Website
            APP_DEEP_LINK (4):
                App Deep Link
            APP_STORE (5):
                iOS App Store or Play Store
            PHONE_CALL (6):
                Call Dialer
            MAP_DIRECTIONS (7):
                Map App
            LOCATION_LISTING (8):
                Location Dedicated Page
            MESSAGE (9):
                Text Message
            LEAD_FORM (10):
                Lead Generation Form
            YOUTUBE (11):
                YouTube
            UNMODELED_FOR_CONVERSIONS (12):
                Ad Destination for Conversions with keys
                unknown
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        NOT_APPLICABLE = 2
        WEBSITE = 3
        APP_DEEP_LINK = 4
        APP_STORE = 5
        PHONE_CALL = 6
        MAP_DIRECTIONS = 7
        LOCATION_LISTING = 8
        MESSAGE = 9
        LEAD_FORM = 10
        YOUTUBE = 11
        UNMODELED_FOR_CONVERSIONS = 12


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.common.types import policy
from google.ads.googleads.v22.enums.types import asset_link_primary_status
from google.ads.googleads.v22.enums.types import (
    asset_link_primary_status_reason,
)
from google.ads.googleads.v22.enums.types import (
    asset_offline_evaluation_error_reasons,
)
from google.ads.googleads.v22.enums.types import policy_approval_status
from google.ads.googleads.v22.enums.types import policy_review_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.common",
    marshal="google.ads.googleads.v22",
    manifest={
        "AdAssetPolicySummary",
        "AssetLinkPrimaryStatusDetails",
        "AssetDisapproved",
    },
)


class AdAssetPolicySummary(proto.Message):
    r"""Contains policy information for an asset inside an ad.

    Attributes:
        policy_topic_entries (MutableSequence[google.ads.googleads.v22.common.types.PolicyTopicEntry]):
            The list of policy findings for this asset.
        review_status (google.ads.googleads.v22.enums.types.PolicyReviewStatusEnum.PolicyReviewStatus):
            Where in the review process this asset.
        approval_status (google.ads.googleads.v22.enums.types.PolicyApprovalStatusEnum.PolicyApprovalStatus):
            The overall approval status of this asset,
            which is calculated based on the status of its
            individual policy topic entries.
    """

    policy_topic_entries: MutableSequence[policy.PolicyTopicEntry] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=policy.PolicyTopicEntry,
        )
    )
    review_status: (
        policy_review_status.PolicyReviewStatusEnum.PolicyReviewStatus
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=policy_review_status.PolicyReviewStatusEnum.PolicyReviewStatus,
    )
    approval_status: (
        policy_approval_status.PolicyApprovalStatusEnum.PolicyApprovalStatus
    ) = proto.Field(
        proto.ENUM,
        number=3,
        enum=policy_approval_status.PolicyApprovalStatusEnum.PolicyApprovalStatus,
    )


class AssetLinkPrimaryStatusDetails(proto.Message):
    r"""Provides the detail of a PrimaryStatus. Each asset link has a
    PrimaryStatus value (e.g. NOT_ELIGIBLE, meaning not serving), and
    list of corroborating PrimaryStatusReasons (e.g.
    [ASSET_DISAPPROVED]). Each reason may have some additional details
    annotated with it. For instance, when the reason is
    ASSET_DISAPPROVED, the details field will contain additional
    information about the offline evaluation errors which led to the
    asset being disapproved.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        reason (google.ads.googleads.v22.enums.types.AssetLinkPrimaryStatusReasonEnum.AssetLinkPrimaryStatusReason):
            Provides the reason of this PrimaryStatus.

            This field is a member of `oneof`_ ``_reason``.
        status (google.ads.googleads.v22.enums.types.AssetLinkPrimaryStatusEnum.AssetLinkPrimaryStatus):
            Provides the PrimaryStatus of this status
            detail.

            This field is a member of `oneof`_ ``_status``.
        asset_disapproved (google.ads.googleads.v22.common.types.AssetDisapproved):
            Provides the details for
            AssetLinkPrimaryStatusReason.ASSET_DISAPPROVED

            This field is a member of `oneof`_ ``details``.
    """

    reason: (
        asset_link_primary_status_reason.AssetLinkPrimaryStatusReasonEnum.AssetLinkPrimaryStatusReason
    ) = proto.Field(
        proto.ENUM,
        number=1,
        optional=True,
        enum=asset_link_primary_status_reason.AssetLinkPrimaryStatusReasonEnum.AssetLinkPrimaryStatusReason,
    )
    status: (
        asset_link_primary_status.AssetLinkPrimaryStatusEnum.AssetLinkPrimaryStatus
    ) = proto.Field(
        proto.ENUM,
        number=2,
        optional=True,
        enum=asset_link_primary_status.AssetLinkPrimaryStatusEnum.AssetLinkPrimaryStatus,
    )
    asset_disapproved: "AssetDisapproved" = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="details",
        message="AssetDisapproved",
    )


class AssetDisapproved(proto.Message):
    r"""Details related to AssetLinkPrimaryStatusReasonPB.ASSET_DISAPPROVED

    Attributes:
        offline_evaluation_error_reasons (MutableSequence[google.ads.googleads.v22.enums.types.AssetOfflineEvaluationErrorReasonsEnum.AssetOfflineEvaluationErrorReasons]):
            Provides the quality evaluation disapproval
            reason of an asset.
    """

    offline_evaluation_error_reasons: MutableSequence[
        asset_offline_evaluation_error_reasons.AssetOfflineEvaluationErrorReasonsEnum.AssetOfflineEvaluationErrorReasons
    ] = proto.RepeatedField(
        proto.ENUM,
        number=1,
        enum=asset_offline_evaluation_error_reasons.AssetOfflineEvaluationErrorReasonsEnum.AssetOfflineEvaluationErrorReasons,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "PositiveGeoTargetTypeEnum",
    },
)


class PositiveGeoTargetTypeEnum(proto.Message):
    r"""Container for enum describing possible positive geo target
    types.

    """

    class PositiveGeoTargetType(proto.Enum):
        r"""The possible positive geo target types.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                The value is unknown in this version.
            PRESENCE_OR_INTEREST (5):
                Specifies that an ad is triggered if the user
                is in, or shows interest in, advertiser's
                targeted locations.
            SEARCH_INTEREST (6):
                Specifies that an ad is triggered if the user
                searches for advertiser's targeted locations.
                This can only be used with Search and standard
                Shopping campaigns.
            PRESENCE (7):
                Specifies that an ad is triggered if the user
                is in or regularly in advertiser's targeted
                locations.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        PRESENCE_OR_INTEREST = 5
        SEARCH_INTEREST = 6
        PRESENCE = 7


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example generates historical metrics for keyword planning.

For more details see this guide:
https://developers.google.com/google-ads/api/docs/keyword-planning/generate-historical-metrics
"""

from typing import Iterable
import argparse
import sys

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.common.types.keyword_plan_common import (
    KeywordPlanHistoricalMetrics,
    MonthlySearchVolume,
)
from google.ads.googleads.v22.services.services.google_ads_service.client import (
    GoogleAdsServiceClient,
)
from google.ads.googleads.v22.services.services.keyword_plan_idea_service.client import (
    KeywordPlanIdeaServiceClient,
)
from google.ads.googleads.v22.services.types.keyword_plan_idea_service import (
    GenerateKeywordHistoricalMetricsRequest,
    GenerateKeywordHistoricalMetricsResponse,
    GenerateKeywordHistoricalMetricsResult,
)


# [START generate_historical_metrics]
def main(client: GoogleAdsClient, customer_id: str):
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
    """
    generate_historical_metrics(client, customer_id)


def generate_historical_metrics(client: GoogleAdsClient, customer_id: str):
    """Generates historical metrics and prints the results.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
    """
    googleads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )
    keyword_plan_idea_service: KeywordPlanIdeaServiceClient = (
        client.get_service("KeywordPlanIdeaService")
    )
    request: GenerateKeywordHistoricalMetricsRequest = client.get_type(
        "GenerateKeywordHistoricalMetricsRequest"
    )
    request.customer_id = customer_id
    request.keywords = ["mars cruise", "cheap cruise", "jupiter cruise"]
    # Geo target constant 2840 is for USA.
    request.geo_target_constants.append(
        googleads_service.geo_target_constant_path("2840")
    )
    request.keyword_plan_network = (
        client.enums.KeywordPlanNetworkEnum.GOOGLE_SEARCH
    )
    # Language criteria 1000 is for English. For the list of language criteria
    # IDs, see:
    # https://developers.google.com/google-ads/api/reference/data/codes-formats#languages
    request.language = googleads_service.language_constant_path("1000")

    response: GenerateKeywordHistoricalMetricsResponse = (
        keyword_plan_idea_service.generate_keyword_historical_metrics(
            request=request
        )
    )

    results: Iterable[GenerateKeywordHistoricalMetricsResult] = response.results
    for result in results:
        metrics: KeywordPlanHistoricalMetrics = result.keyword_metrics
        # These metrics include those for both the search query and any variants
        # included in the response.
        print(
            f"The search query '{result.text}' (and the following variants: "
            f"'{result.close_variants if result.close_variants else 'None'}'), "
            "generated the following historical metrics:\n"
        )

        # Approximate number of monthly searches on this query averaged for the
        # past 12 months.
        print(f"\tApproximate monthly searches: {metrics.avg_monthly_searches}")

        # The competition level for this search query.
        print(f"\tCompetition level: {metrics.competition}")

        # The competition index for the query in the range [0, 100]. This shows
        # how competitive ad placement is for a keyword. The level of
        # competition from 0-100 is determined by the number of ad slots filled
        # divided by the total number of ad slots available. If not enough data
        # is available, undef will be returned.
        print(f"\tCompetition index: {metrics.competition_index}")

        # Top of page bid low range (20th percentile) in micros for the keyword.
        print(
            f"\tTop of page bid low range: {metrics.low_top_of_page_bid_micros}"
        )

        # Top of page bid high range (80th percentile) in micros for the
        # keyword.
        print(
            "\tTop of page bid high range: "
            f"{metrics.high_top_of_page_bid_micros}"
        )

        # Approximate number of searches on this query for the past twelve
        # months.
        months: Iterable[MonthlySearchVolume] = metrics.monthly_search_volumes
        for month in months:
            print(
                f"\tApproximately {month.monthly_searches} searches in "
                f"{month.month.name}, {month.year}"
            )
            # [END generate_historical_metrics]


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Generates forecast metrics for keyword planning."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )

    args = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client = GoogleAdsClient.load_from_storage(version="v22")

    try:
        main(googleads_client, args.customer_id)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'Error with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



#!/usr/bin/env python
#
# Copyright 2018 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""This example gets a list of which resources have been changed in an account."""


import argparse
import sys

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.services.services.google_ads_service.client import (
    GoogleAdsServiceClient,
)
from google.ads.googleads.v22.services.types.google_ads_service import (
    SearchGoogleAdsRequest,
    SearchPagedResponse,
    GoogleAdsRow,
)
from google.ads.googleads.v22.resources.types.change_status import ChangeStatus


# [START get_change_summary]
def main(client: GoogleAdsClient, customer_id: str) -> None:
    ads_service: GoogleAdsServiceClient = client.get_service("GoogleAdsService")

    # Construct a query to find information about changed resources in your
    # account.
    query = """
        SELECT
          change_status.resource_name,
          change_status.last_change_date_time,
          change_status.resource_type,
          change_status.campaign,
          change_status.ad_group,
          change_status.resource_status,
          change_status.ad_group_ad,
          change_status.ad_group_criterion,
          change_status.campaign_criterion
        FROM change_status
        WHERE change_status.last_change_date_time DURING LAST_14_DAYS
        ORDER BY change_status.last_change_date_time
        LIMIT 10000"""

    search_request: SearchGoogleAdsRequest = client.get_type(
        "SearchGoogleAdsRequest"
    )
    search_request.customer_id = customer_id
    search_request.query = query

    response: SearchPagedResponse = ads_service.search(request=search_request)

    row: GoogleAdsRow
    for row in response:
        cs: ChangeStatus = row.change_status
        resource_type: str = cs.resource_type.name
        resource_name: str
        if resource_type == "AD_GROUP":
            resource_name = cs.ad_group
        elif resource_type == "AD_GROUP_AD":
            resource_name = cs.ad_group_ad
        elif resource_type == "AD_GROUP_CRITERION":
            resource_name = cs.ad_group_criterion
        elif resource_type == "CAMPAIGN":
            resource_name = cs.campaign
        elif resource_type == "CAMPAIGN_CRITERION":
            resource_name = cs.campaign_criterion
        else:
            resource_name = "UNKNOWN"

        resource_status: str = cs.resource_status.name
        print(
            f"On '{cs.last_change_date_time}', change status "
            f"'{cs.resource_name}' shows that a resource type of "
            f"'{resource_type}' with resource name '{resource_name}' was "
            f"{resource_status}"
        )
        # [END get_change_summary]


if __name__ == "__main__":
    # GoogleAdsClient will read a google-ads.yaml configuration file in the
    parser = argparse.ArgumentParser(
        description=(
            "Displays account changes that occurred in the last 7 days."
        )
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    args = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client = GoogleAdsClient.load_from_storage(version="v22")

    try:
        main(googleads_client, args.customer_id)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "LandingPageSourceEnum",
    },
)


class LandingPageSourceEnum(proto.Message):
    r"""Container for enum describing the source of a landing page in
    the landing page report.

    """

    class LandingPageSource(proto.Enum):
        r"""The source of a landing page in the landing page report.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            ADVERTISER (2):
                The landing page was explicitly provided by
                the advertiser.
            AUTOMATIC (3):
                The landing page was selected automatically.
                This could happen when the advertiser enables AI
                Max or other features that automatically select
                landing pages and Google selects the best
                landing page for the query.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        ADVERTISER = 2
        AUTOMATIC = 3


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.enums.types import (
    response_content_type as gage_response_content_type,
)
from google.ads.googleads.v22.resources.types import batch_job
from google.ads.googleads.v22.services.types import google_ads_service
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.services",
    marshal="google.ads.googleads.v22",
    manifest={
        "MutateBatchJobRequest",
        "BatchJobOperation",
        "MutateBatchJobResponse",
        "MutateBatchJobResult",
        "RunBatchJobRequest",
        "AddBatchJobOperationsRequest",
        "AddBatchJobOperationsResponse",
        "ListBatchJobResultsRequest",
        "ListBatchJobResultsResponse",
        "BatchJobResult",
    },
)


class MutateBatchJobRequest(proto.Message):
    r"""Request message for
    [BatchJobService.MutateBatchJob][google.ads.googleads.v22.services.BatchJobService.MutateBatchJob].

    Attributes:
        customer_id (str):
            Required. The ID of the customer for which to
            create a batch job.
        operation (google.ads.googleads.v22.services.types.BatchJobOperation):
            Required. The operation to perform on an
            individual batch job.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operation: "BatchJobOperation" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="BatchJobOperation",
    )


class BatchJobOperation(proto.Message):
    r"""A single operation on a batch job.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        create (google.ads.googleads.v22.resources.types.BatchJob):
            Create operation: No resource name is
            expected for the new batch job.

            This field is a member of `oneof`_ ``operation``.
        remove (str):
            Remove operation: The batch job must not have been run. A
            resource name for the removed batch job is expected, in this
            format:

            ``customers/{customer_id}/batchJobs/{batch_job_id}``

            This field is a member of `oneof`_ ``operation``.
    """

    create: batch_job.BatchJob = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="operation",
        message=batch_job.BatchJob,
    )
    remove: str = proto.Field(
        proto.STRING,
        number=4,
        oneof="operation",
    )


class MutateBatchJobResponse(proto.Message):
    r"""Response message for
    [BatchJobService.MutateBatchJob][google.ads.googleads.v22.services.BatchJobService.MutateBatchJob].

    Attributes:
        result (google.ads.googleads.v22.services.types.MutateBatchJobResult):
            The result for the mutate.
    """

    result: "MutateBatchJobResult" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="MutateBatchJobResult",
    )


class MutateBatchJobResult(proto.Message):
    r"""The result for the batch job mutate.

    Attributes:
        resource_name (str):
            The resource name of the batch job.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class RunBatchJobRequest(proto.Message):
    r"""Request message for
    [BatchJobService.RunBatchJob][google.ads.googleads.v22.services.BatchJobService.RunBatchJob].

    Attributes:
        resource_name (str):
            Required. The resource name of the BatchJob
            to run.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class AddBatchJobOperationsRequest(proto.Message):
    r"""Request message for
    [BatchJobService.AddBatchJobOperations][google.ads.googleads.v22.services.BatchJobService.AddBatchJobOperations].

    Attributes:
        resource_name (str):
            Required. The resource name of the batch job.
        sequence_token (str):
            A token used to enforce sequencing.

            The first AddBatchJobOperations request for a batch job
            should not set sequence_token. Subsequent requests must set
            sequence_token to the value of next_sequence_token received
            in the previous AddBatchJobOperations response.
        mutate_operations (MutableSequence[google.ads.googleads.v22.services.types.MutateOperation]):
            Required. The list of mutates being added.

            Operations can use negative integers as temp ids
            to signify dependencies between entities created
            in this batch job. For example, a customer with
            id = 1234 can create a campaign and an ad group
            in that same campaign by creating a campaign in
            the first operation with the resource name
            explicitly set to "customers/1234/campaigns/-1",
            and creating an ad group in the second operation
            with the campaign field also set to
            "customers/1234/campaigns/-1".
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    sequence_token: str = proto.Field(
        proto.STRING,
        number=2,
    )
    mutate_operations: MutableSequence[google_ads_service.MutateOperation] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=3,
            message=google_ads_service.MutateOperation,
        )
    )


class AddBatchJobOperationsResponse(proto.Message):
    r"""Response message for
    [BatchJobService.AddBatchJobOperations][google.ads.googleads.v22.services.BatchJobService.AddBatchJobOperations].

    Attributes:
        total_operations (int):
            The total number of operations added so far
            for this batch job.
        next_sequence_token (str):
            The sequence token to be used when calling
            AddBatchJobOperations again if more operations need to be
            added. The next AddBatchJobOperations request must set the
            sequence_token field to the value of this field.
    """

    total_operations: int = proto.Field(
        proto.INT64,
        number=1,
    )
    next_sequence_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class ListBatchJobResultsRequest(proto.Message):
    r"""Request message for
    [BatchJobService.ListBatchJobResults][google.ads.googleads.v22.services.BatchJobService.ListBatchJobResults].

    Attributes:
        resource_name (str):
            Required. The resource name of the batch job
            whose results are being listed.
        page_token (str):
            Token of the page to retrieve. If not specified, the first
            page of results will be returned. Use the value obtained
            from ``next_page_token`` in the previous response in order
            to request the next page of results.
        page_size (int):
            Number of elements to retrieve in a single page. The default
            and the page_size limit is 1000. A value of 0 or an unset
            page size will be defaulted to 1000. A page size less than 0
            or greater than 1000 will result in an INVALID_PAGE_SIZE
            error.
        response_content_type (google.ads.googleads.v22.enums.types.ResponseContentTypeEnum.ResponseContentType):
            The response content type setting. Determines
            whether the mutable resource or just the
            resource name should be returned.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=3,
    )
    response_content_type: (
        gage_response_content_type.ResponseContentTypeEnum.ResponseContentType
    ) = proto.Field(
        proto.ENUM,
        number=4,
        enum=gage_response_content_type.ResponseContentTypeEnum.ResponseContentType,
    )


class ListBatchJobResultsResponse(proto.Message):
    r"""Response message for
    [BatchJobService.ListBatchJobResults][google.ads.googleads.v22.services.BatchJobService.ListBatchJobResults].

    Attributes:
        results (MutableSequence[google.ads.googleads.v22.services.types.BatchJobResult]):
            The list of rows that matched the query.
        next_page_token (str):
            Pagination token used to retrieve the next page of results.
            Pass the content of this string as the ``page_token``
            attribute of the next request. ``next_page_token`` is not
            returned for the last page.
    """

    @property
    def raw_page(self):
        return self

    results: MutableSequence["BatchJobResult"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="BatchJobResult",
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class BatchJobResult(proto.Message):
    r"""An individual batch job result.

    Attributes:
        operation_index (int):
            Index of the mutate operation.
        mutate_operation_response (google.ads.googleads.v22.services.types.MutateOperationResponse):
            Response for the mutate.
            May be empty if errors occurred.
        status (google.rpc.status_pb2.Status):
            Details of the errors when processing the
            operation.
    """

    operation_index: int = proto.Field(
        proto.INT64,
        number=1,
    )
    mutate_operation_response: google_ads_service.MutateOperationResponse = (
        proto.Field(
            proto.MESSAGE,
            number=2,
            message=google_ads_service.MutateOperationResponse,
        )
    )
    status: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=3,
        message=status_pb2.Status,
    )


__all__ = tuple(sorted(__protobuf__.manifest))
